<?php
/*
 * Public Domain
 */
if (!defined('NOT_ACCESSED_DIRECTLY')) exit('This file may not be directly accessed');

//Your module's directory needs to contain a PHP file named module_code.php. This file
//should contain a PHP class definition, which should contain your module's program
//code.

//The class name you use MUST match your module's directory name.

//Your class should either extend the module_base_class, or else extend another module
//that extends the module_base_class.
//If you extend another module, you must list that module as a dependancy in your
//description.yaml file. The CMS will then check if the module you are extending can be
//run and include it for you each time the code your module is included; you do not
//need to use "include" or "require" in your code to include the module you are
//extending.


class example_module_4 extends module_base_class {
	
	//Your fillOrganizerPanel() method is called by the CMS whenever an Admin visits
	//one of your panels in Storekeeper.
	public function fillOrganizerPanel($path, &$panel, $refinerName, $refinerId, $mode) {
		
		//You should always use an if () or a switch () statement on the path,
		//to ensure that you are running code for the correct Panel.
		switch ($path) {
			case 'example_module_4/panels/animals':
				
				//Loop through each row, and crop the description if it is too long
				foreach ($panel['items'] as &$item) {
					if (strlen($item['description']) > 50) {
						$item['description'] =
							substr($item['description'], 0, 50). '...';
					}
				}
				
				break;
		}
	}
	
	//Your handleOrganizerPanelAJAX() method is called by the CMS whenever an Admin presses
	//one of your buttons that has the ajax property set.
	public function handleOrganizerPanelAJAX($path, $ids, $ids2, $refinerName, $refinerId) {
		
		//You should always use an if () or a switch () statement on the path,
		//to ensure that you are running code for the correct Panel.
		switch ($path) {
			case 'example_module_4/panels/animals':
				
				//Handle the case where an Admin presses the delete button.
				if (post('action') == 'delete_animal') {
					//Check to see if this Admin is actually allowed to delete an
					//animal. You can check a permission by calling the checkPriv()
					//function with the name of the permission that you wish to check.
				 	if (checkPriv('_PRIV_EXAMPLE_DELETE_ANIMAL')) {
						foreach (explode(',', $ids) as $id) {
							example_module_4::deleteAnimal($id);
						}
					}
				}
				
				break;
		}
	}
	
	//Your fillAdminBox() method is called by the CMS whenever an Admin opens one of
	//your Admin Boxes, or an Admin Box that you have added new fields into.
	public function fillAdminBox($path, $settingGroup, &$box, &$fields, &$values) {
		
		//You should always use an if () or a switch () statement on the path,
		//to ensure that you are running code for the correct Admin Box.
		switch ($path) {
			case 'example_module_4__animal':
				//If this box is opened with an id set, load the details
				if ($box['key']['id']) {
					$details = example_module_4::animalDetails($box['key']['id']);
					
					//Set a title
					$box['title'] = adminPhrase('Viewing/Editing the Animal "[[name]]"', $details);
					
					//The $values array contains references to the values of the fields.
					//E.g. to set the value of a field we can do:
					$values['details/name'] = $details['name'];
					
					//Here, "details" is the name of the tab, "name" is the name of the field, and
						//$values['details/name']
					//is actually a reference to 
						//$box['tabs']['details']['fields']['name']['value']
					
					//If the field names are unique, you do not need to specify the name of the tab.
					//E.g. to set the value of a field we can also do:
					$values['description'] = $details['description'];
					
					//Here, "description" is the name of the field, and
						//$values['description']
					//is actually a reference to 
						//$box['tabs']['details']['fields']['description']['value']
					
				}
				
				break;
		}
	}
	
	//Your validateAdminBox() method is called by the CMS whenever an Admin tries to
	//save one of your Admin Boxes, or when they try to switch to a different tab.
	//This allows you to add custom validation to your fields.
	public function validateAdminBox($path, $settingGroup, &$box, &$fields, &$values, $changes, $saving) {
		
		//You should always use an if () or a switch () statement on the path,
		//to ensure that you are running code for the correct Admin Box.
		switch ($path) {
			case 'example_module_4__animal':
				
				//Only do validation logic on a tab if it is in 'Edit Mode'.
				if (!empty($box['tabs']['details']['edit_mode']['on'])) {
					
					//Call the isInvalidAnimal() function to get any errors in the submission
					if ($e = example_module_4::isInvalidAnimal($values, $box['key']['id'])) {
						
						//If there are errors, add them to the first tab
						foreach ($e->errors as $error) {
							$box['tabs']['details']['errors'][] = adminPhrase($error);
						}
					}
				}
				
				break;
		}
	}
	
	//Your saveAdminBox() method is called by the CMS whenever an Admin tries to
	//save one of your Admin Boxes. You should create/update any rows in the database
	//that you need to save the information that an Admin as entered.
	public function saveAdminBox($path, $settingGroup, &$box, &$fields, &$values, $changes) {
		
		//You should always use an if () or a switch () statement on the path,
		//to ensure that you are running code for the correct Admin Box.
		switch ($path) {
			case 'example_module_4__animal':
				if (!empty($box['tabs']['details']['edit_mode']['on'])) {
					
					//Update an existing animal
					if ($box['key']['id']) {
						
						//Remember to check permissions
						exitIfNotCheckPriv('_PRIV_EXAMPLE_EDIT_ANIMAL');
						
						//Call the API function to save the details
						example_module_4::saveAnimal($values, $box['key']['id']);
					
					//Create a new animal
					} else {
						
						//Remember to check permissions
						exitIfNotCheckPriv('_PRIV_EXAMPLE_CREATE_ANIMAL');
						
						//Call the API function to save the details.
						//After we are done saving, we need to put the newly created
						//id into $box['key']['id'].
						$box['key']['id'] = example_module_4::saveAnimal($values);
					}
				}
				
				break;
		}
	}
	
	
	
	
	//An API function that other Modules can call to get the name and description of
	//an animal.
	public static function animalDetails($id) {
		return
			getRow(
				EXAMPLE_MODULE_4_PREFIX. 'animals',
				array('name', 'description'),
				array('id' => $id));
	}
	
	//An API function that other Modules can call to check if an animal name is
	//already taken.
	public static function checkAnimalNameUnique($name, $currentId = false) {
		$existingId =
			getRow(
				EXAMPLE_MODULE_4_PREFIX. 'animals',
				'id',
				array('name' => $name));
		
		return !$existingId || ($currentId && $currentId == $existingId);
	}
	
	//An API function that other Modules can call to delete an animal.
	public static function deleteAnimal($id) {
		sendSignal('exampleModuleEventAnimalDeleted', array('id' => $id));
		deleteRow(EXAMPLE_MODULE_4_PREFIX. 'animals', $id);
	}
	
	
	//An API function to check if an animal is valid.
	//To avoid code duplication it's implemented by calling the saveAnimal() function
	//with $doSave set to false.
	public static function isInvalidAnimal($values, $id = false) {
		return example_module_4::saveAnimal($values, $id, false);
	}
	
	//An API function to save an animal to the database.
	//It will only save it if it passes a validation check; if it is not valid then this
	//function will return an error object.
	public static function saveAnimal($values, $id = false, $doSave = true) {
		$details = array();
		
		//First, validate the submission.
		$e = new zenario_error();
		
		//Validate the name field if it is set.
		//(Always validate it when creating a new animal.)
		if (!$id || isset($values['name'])) {
			//Check that a name was entered...
			if (empty($values['name'])) {
				$e->add('name', '_ERROR_ANIMAL_NAME_INCOMPLETE');
			
			//...has no special characters...
			} elseif (!validateScreenName($values['name'])) {
				$e->add('name', '_ERROR_ANIMAL_NAME_INVALID');
			
			//...and is not already taken by a different row.
			} elseif (!example_module_4::checkAnimalNameUnique($values['name'], $id)) {
				$e->add('name', '_ERROR_ANIMAL_NAME_IN_USE');
			
			} else {
				$details['name'] = $values['name'];
			}
		}
		
		//Validate the description field if it is set.
		//(Always validate it when creating a new animal.)
		if (!$id || isset($values['description'])) {
			//Check that a description was entered.
			if (empty($values['description'])) {
				$e->add('description', '_ERROR_ANIMAL_DESCRIPTION_INCOMPLETE');
			
			} else {
				$details['description'] = $values['description'];
			}
		}
		
		//If there were errors, return the errors
		if (!empty($e->errors)) {
			return $e;
		
		//If we were just validating, stop at this point
		} elseif (!$doSave) {
			return false;
		
		} else {
			//Save the details to the database
			$newId = setRow(EXAMPLE_MODULE_4_PREFIX. 'animals', $details, $id);
			
			//Send a signal to let other Modules know this event has happened
			if ($id) {
				sendSignal(
					'exampleModuleEventAnimalModified',
					array('id' => $id));
			
			} else {
				sendSignal(
					'exampleModuleEventAnimalCreated',
					array('id' => $newId));
			}
			
			//Return the primary id from the database to the caller
			return $newId;
		}
	}
	
	// This static function is used to pass a list of animals to datasets
	// so you can select a list of animals as a centralised list
	public static function getAnimals($mode, $value = false) {
		switch ($mode) {
			case ZENARIO_CENTRALISED_LIST_MODE_INFO:
				return array('can_filter' => false);
			case ZENARIO_CENTRALISED_LIST_MODE_LIST:
				return getRowsArray(EXAMPLE_MODULE_4_PREFIX. 'animals', 'name', array(), 'name');
			case ZENARIO_CENTRALISED_LIST_MODE_VALUE:
				return example_module_4::animalDetails($value);
		}
	}
}